@@ -3,13 +3,13 @@ package Kelp::Routes;
 use Carp;
 
 use Kelp::Base;
-use Kelp::Routes::Pattern;
 use Plack::Util;
 use Class::Inspector;
 
-attr base    => '';
-attr routes  => sub { [] };
-attr names   => sub { {} };
+attr base        => '';
+attr pattern_obj => 'Kelp::Routes::Pattern';
+attr routes      => sub { [] };
+attr names       => sub { {} };
 
 # Cache
 attr _CACHE => sub { {} };
@@ -119,7 +119,7 @@ sub _parse_route {
     }
 
     # Create pattern object
-    push @{ $self->routes }, Kelp::Routes::Pattern->new(%$val);
+    push @{ $self->routes }, $self->build_pattern( $val );
 
     # Add route index to names
     if ( my $name = $val->{name} ) {
@@ -135,6 +135,14 @@ sub _parse_route {
     }
 }
 
+# Override to use a custom pattern object
+sub build_pattern {
+    my ( $self, $args ) = @_;
+    my $package = $self->pattern_obj;
+    eval qq{require $package};
+    return $package->new( %$args );
+}
+
 sub url {
     my $self = shift;
     my $name = shift // die "Route name is missing";
@@ -171,12 +179,11 @@ sub dispatch {
     $route || die "No route pattern instance supplied";
 
     # Shortcuts
-    my $req = $app->req;
     my $to  = $route->to;
 
     # Destination must be either a scalar, or a code reference
-    if ( !$to || ref $to && ref $to ne 'CODE' ) {
-        die 'Invalid destination for ' . $req->path;
+    if ( !$to || ( ref $to && ref $to ne 'CODE' ) ) {
+        die 'Invalid destination for ' . $app->req->path;
     }
 
     # If the destination is not a code reference, then we assume it's
@@ -185,13 +192,14 @@ sub dispatch {
 
         # Check if the destination function exists
         unless ( exists &$to ) {
-            die sprintf( 'Route not found %s for %s', $to, $req->path );
+            die sprintf( 'Route not found %s for %s', $to, $app->req->path );
         }
 
         # Move to reference
         $to = \&{$to};
     }
 
+    $app->before_dispatch( $route->to );
     return $to->( $app, @{ $route->param } );
 }
 
@@ -241,8 +249,6 @@ Then you add your application's routes and their descriptions:
     $r->add( '/path' => 'Module::function' );
     ...
 
-=cut
-
 =item B<Matching>
 
 Once you have your routes added, you can match with the L</match> subroutine.
@@ -253,8 +259,6 @@ The Kelp framework already does matching for you, so you may never
 have to do your own matching. The above example is provided only for
 reference.
 
-=cut
-
 =item B<Building URLs from routes>
 
 You can name each of your routes, and use that name later to build a URL:
@@ -265,8 +269,6 @@ You can name each of your routes, and use that name later to build a URL:
 This can be used in views and other places where you need the full URL of
 a route.
 
-=cut
-
 =back
 
 =head1 PLACEHOLDERS
@@ -388,7 +390,7 @@ specified path. If that route is marked as a bridge, then L</match> will
 continue looking for another match, and will eventually return an array of one or
 more routes. Bridges can be used for authentication or other route preprocessing.
 
-    $r->add( '/users', { to => 'Users::auth', bridge => 1 } );
+    $r->add( '/users/*', { to => 'Users::auth', bridge => 1 } );
     $r->add( '/users/:action' => 'Users::dispatch' );
 
 The above example will require F</users/profile> to go through two
@@ -403,7 +405,7 @@ A quick way to add bridges is to use the L</tree> option. It allows you to
 define all routes under a bridge. Example:
 
     $r->add(
-        '/users' => {
+        '/users/*' => {
             to   => 'users#auth',
             name => 'users',
             tree => [
@@ -438,8 +440,6 @@ parent, so the following five new routes are created:
     /users/settings/email   -> MyApp::Users::email
     /users/settings/login   -> MyApp::Users::login
 
-=cut
-
 =item
 
 The names of the routes are joined via C<_> with the name of their parent:
@@ -450,15 +450,11 @@ The names of the routes are joined via C<_> with the name of their parent:
     /users/settings/email   -> 'users_settings_email'
     /users/settings/login   -> 'users_settings_login'
 
-=cut
-
 =item
 
 The C</users> and C</users/settings> routes are automatically marked as
 bridges, because they contain a tree.
 
-=cut
-
 =back
 
 =head1 ATTRIBUTES
@@ -497,11 +493,17 @@ faster. The C<cache> attribute can optionally be initialized with an instance of
 a caching module with interface similar to L<CHI> and L<Cache>.
 The module interface should at the very least provide the following methods:
 
-=head3 get($key) - retrieve a key from the cache
+=head3 get($key)
 
-=head3 set($key, $value, $expiration) - set a key in the cache
+retrieve a key from the cache
 
-=head3 clear() - clear all cache
+=head3 set($key, $value, $expiration)
+
+set a key in the cache
+
+=head3 clear()
+
+clear all cache
 
 The caching module should be initialized in the config file:
 
@@ -540,16 +542,12 @@ C<"users#item"> becomes C<"Users::item">.
 
     $r->add( '/home' => 'user#home' );
 
-=cut
-
 =item
 
 A code reference.
 
     $r->add( '/system' => sub { return \%ENV } );
 
-=cut
-
 =item
 
 A hashref with options.
@@ -564,8 +562,6 @@ A hashref with options.
 
 See L</Destination Options> for details.
 
-=cut
-
 =back
 
 =head3 Destination Options
@@ -648,12 +644,18 @@ Set default values for optional placeholders.
 
 =head4 bridge
 
-If set to one this route will be treated as a bridge. Please see L</bridges>
+If set to 1 this route will be treated as a bridge. Please see L</BRIDGES>
 for more information.
 
 =head4 tree
 
-Creates a tree of sub-routes. See L</trees> for more information and examples.
+Creates a tree of sub-routes. See L</TREES> for more information and examples.
+
+=head2 url
+
+    my $url = $r->url($path, @arguments);
+
+Builds an url from path and arguments. If the request is named a name can be specified instead.
 
 =head2 match
 
@@ -673,6 +675,12 @@ Routes that used regular expressions instead of patterns will only initialize
 the C<param> array with the regex captures, unless those patterns are using
 named captures in which case the C<named> hash will also be initialized.
 
+=head2 dispatch
+
+    my $result = $r->dispatch($kelp, $route_pattern);
+
+Dispatches an instance of L<Kelp::Routes::Pattern> by running the route destination specified in L<Kelp::Routes::Pattern/to>
+
 =head1 EXTENDING
 
 This is the default router class for each new Kelp application, but it doesn't
@@ -687,3 +695,4 @@ the application instance into a controller class.
 This module was inspired by L<Routes::Tiny>.
 
 =cut
+
