@@ -10,12 +10,14 @@ sub dispatch {
 
     my $to = $match->to or croak 'No destination defined';
 
-    return $self->SUPER::dispatch($match, $app) if ref $to;
+    return $self->SUPER::dispatch($app, $match) if ref $to;
 
     my ($controller_class, $action) = ($to =~ /^(.+)::(\w+)$/)
         or croak "Invalid controller '$to'";
 
     my $controller = $app->_clone($controller_class);
+
+    $controller->before_dispatch($action);
     return $controller->$action(@{ $match->param });
 }
 
@@ -59,7 +61,7 @@ Kelp::Routes::Controller - Routes and controller for Kelp
     package MyApp::Controller::Users;
     use Kelp::Base 'MyApp::Controller';
 
-    my read {
+    sub read {
         my $self = shift;   # $self is an instance of 'MyApp::Controller::Users'
         ...
     }
@@ -78,3 +80,4 @@ functionality.
 
 
 =cut
+
